%% setting of the system
function [sysInfo,fmDataInfo]  = GradDiss_settings(potential_type,scheme)

% full model settings
dimX = 2;
mu1 = 1; beta = 2; mu2=1;
%potential_type = 'GradDiss';    % 'A1B1'
switch potential_type
    case 'GradDiss' 
        fnA         = @(x) mu1/2*exp(x.^2);    fnB   = @(x) mu2/2*(x.^2);
        gradA       = @(x) mu1*x.*exp(x.^2);   gradB = @(x) mu2*(x);
        potential_V = @(x,y) fnA(x)+ fnB(y);
        % need to be corrected in high dimension case
        hessA       = @(x)  mu1*((1+2*(x.^2)).*exp(x.^2));
        hessB       = mu2;        
        dVx         = @(x) gradA(x);  % note the .* OK for matrix
        dVy         = @(y) gradB(y);
        % need to be corrected in high dimension case
        dVxx        = @(x) hessA(x);
        dVxy        = 0;
        dVyy        = hessB;
        gradV       = @(X) [dVx(X(1,:)); dVy(X(2,:))];
        hessV       = @(X) [dVxx(X(1,:)), dVxy;  dVxy, dVyy];
        LaplaceGradV= @(X) [mu1*exp(X(1,:).^2).*(6*X(1,:)+4*(X(1,:).^3)); 0];          
end


xgrid = -2:0.1:2; ygrid = -2:0.1:2;
plotPotential_2D(potential_V,xgrid,ygrid);

% information of the true system
sysInfo = struct(...
    'potential_type', potential_type,...
    'potential_V',  potential_V,...
    'dimX',         dimX,...
    'mu1',          mu1,...
    'mu2',          mu2,...
    'beta',         beta,...
    'sigma_1',      sqrt(2/beta),...
    'sigma_2',      sqrt(2/beta),...
    'fnA',          fnA,...
    'fnB',          fnB, ...
    'gradA',        gradA,...
    'gradB',        gradB,...
    'hessA',        hessA,...
    'hessB',        hessB,...
    'xgrid',        xgrid,...
    'ygrid',        ygrid, ...
    'gradV',        gradV,...
    'hessV',        hessV,...
    'LaplaceGradV', LaplaceGradV, ...
    'diffusion',    diag([sqrt(2/beta),sqrt(2/beta)]) ... 
    );

%% setting of the numerical scheme for the true system==>> full model
tN = 1e6; dt = 2e-3; 
Init_distr = 'fixed'; %'Gaussian';
% scheme   = 'SSBE'; %'Split Backward Euler';
ImpSolver= 'Newton';
Newton_maxIte = 40;
Newton_stopCond = dt^2/10;
betaMu = sprintf('beta%ilambda%i_tN%iw',beta,mu1,tN/10000);
fm_type  = [potential_type,'_',betaMu];
fmDataInfo   = struct(...
    'scheme',  scheme,...
    'ImpSolver', ImpSolver, ...
    'dt',      dt,...
    'tN',      tN,...
    'Newton_maxIte', Newton_maxIte,...
    'Newton_stopCond', Newton_stopCond,...
    'type',    fm_type, ... % string for file names
    'dVx',     dVx,...       % used only for XL's code
    'dVy',     dVy, ...
    'dVxx',    dVxx, ...
    'dVxy',    dVxy, ...
    'dVyy',    dVyy, ...
    'betaLambda', betaMu,...
    'getInitial', @(M) getInitial(dimX,M,Init_distr) ... % randn(2,M);
    );
end



function plotPotential_2D(potential_V,xgrid,ygrid)
% plot the potential function

[X,Y] = meshgrid(xgrid,ygrid);
fnVal = potential_V(X,Y);
surf(X,Y,fnVal); colorbar
end
