%% setting of the system
function [sysInfo,fmDataInfo]  = Langevin_x1D_settings(potential_type,scheme)
% 
% full model settings for a damped Langevin dynamics system in 2D

dimX = 1;
% mass matrix
M  = 1;
% potential paremeter; inverse temperature; friction constant
mu = 1; para_k=5; beta = 1; gamma=1; 
%potential_type = 'LangLocal';    % H(x,p)=V(x)+K(p)
switch potential_type
    case 'Langevin_x4sinx' 
        potential_V = @(x) mu/4*(x.^4)+sin(1+para_k*x);
        % need to be corrected in high dimension case        
        dVx         = @(x) mu*(x.^3)+para_k*cos(1+para_k*x);
        dVxx        = @(x) mu*(3*x.^2)-para_k^2*sin(1+para_k*x);
        dVxxx       = @(x) mu*(6*x)-para_k^3*cos(1+para_k*x);      
        
    case 'Langevin_2well'
        mu   = 2; beta = 1; diff_b = sqrt(2/beta); % small mu case
        potential_V = @(x)  mu/4*(x.^4-2*x.^2+1);
        
        dVx         = @(x)  mu*(x.^3-x);
        dVxx        = @(x)  mu*(3*x.^2-1);
        dVxxx       = @(x)  mu*6*x;
end
kinetic_K   = @(p) 1/2*(p.^2);
Hamiltonian_H = @(x,p) potential_V(x)+kinetic_K(p);
dKp         = @(p) p;
dKpp        = 1;

% gradV in fact=-RHS force; RHS = [0, 1; -1, 0]*(grad_H)+[0, 0; 0, -gamma]*[x; p]= Hamiltonian force+friction force
% In the solver function, it needs negative RHS, i.e. negative drift force
gradV       = @(X) -([0, 1; -1, 0]* [dVx(X(1,:)) ; dKp(X(2,:))]+[0, 0;0,-gamma]*[(X(1,:)); (X(2,:))]);
hessV       = @(X) -([0, 1; -1, 0]*([dVxx(X(1,:)),0;0, dKpp] )+[0,0;0,-gamma]);
LaplaceGradV= @(X) -([0, 1; -1, 0]*([dVxxx(X(1,:)),0;0, 0] ));  

xgrid = -2:0.1:2; ygrid = -2:0.1:2;
plotHamiltonian_2D(Hamiltonian_H,xgrid,ygrid);

% information of the true system
sysInfo = struct(...
    'potential_type', potential_type,...
    'Hamiltonian_H',  Hamiltonian_H,...
    'M',              M,...
    'dimX',           dimX,...
    'mu',             mu,...
    'para_k',         para_k,...
    'gamma',          gamma,...
    'beta',           beta,...
    'sigma_1',        0,...
    'sigma_2',        sqrt(2/beta),...
    'xgrid',          xgrid,...
    'ygrid',          ygrid, ...
    'gradV',          gradV,...
    'hessV',          hessV,...
    'LaplaceGradV',   LaplaceGradV, ...
    'diffusion',      diag([0,sqrt(2/beta)]) ... 
    );

%% setting of the numerical scheme for the true system==>> full model
tN = 1e6; dt = 1e-2; 
Init_distr = 'fixed'; %'Gaussian';
% scheme   = 'SSBE'; %'Split Backward Euler';
ImpSolver= 'Newton';
Newton_maxIte = 40;
Newton_stopCond = dt/10;
betaMu = sprintf('beta%ilambda%i_tN%iw',beta,mu, tN/10000);
fm_type  = [potential_type,'_',betaMu];
fmDataInfo   = struct(...
    'scheme',  scheme,...
    'ImpSolver', ImpSolver, ...
    'dt',      dt,...
    'tN',      tN,...
    'Newton_maxIte', Newton_maxIte,...
    'Newton_stopCond', Newton_stopCond,...
    'type',    fm_type, ... % string for file names
    'dVx',     dVx,...       % used only for XL's code
    'dVxx',    dVxx, ...
    'dVxxx',    dVxxx, ...
    'betaLambda', betaMu,...
    'getInitial', @(M) getInitial(dimX,M,Init_distr) ... % randn(2,M);
    );
end



function plotHamiltonian_2D(Hamiltonian_H,xgrid,ygrid)
% plot the potential function

[X,Y] = meshgrid(xgrid,ygrid);
fnVal = Hamiltonian_H(X,Y);
surf(X,Y,fnVal); colorbar
end
