%% Set Parameters for the full system in general form
%               dX = -f(X)dt + b(X) dW 
%  Aim to cover: gradient system, Langevin, Hamiltonian
%{
    Gradient system    dX = -gradV(X)dt + sigma dW; with 
% Xingjie's Entopy Barrier:  V(x,y) = fnA(x) + 
%      dx1 = - grad(fnA)(x1) dt + lambda*<x2-fnB(x1),grad(fnB(x1))> dt + sigma_1 dW_1(t)
%      dx2 = - lambda[x2- B(x1)]dt + sigma_2 dW_2(t)
%}

function [sysInfo,fmDataInfo] = model_settings_all(SAVE_DIR,myslash)
 
scheme   = 'SSBE'; % 'Euler'; 
                    % 'SBE': stochatic Backward Euler
                    % 'SSBE': split-step stochastic Backward Euler
                    % 'BAOAB': time splitting BAOAB method for Hamlitonian system

potential_type = 'LangLocalGradSys';   % select potential type
      % 'Ax2Bsin'   'Ax4Bsin' : entropy Barrier
      % 'x4'                  : x^4; 1D
      % 'GradDiss'            : decoupled: V(x,y)=V1(x)+V2(y)
      % 'GradDissCouple'      : coupled: V(x,y)=V(x,y)
      % 'DoubleWell'          : a*(x.^2-1).^2 + b*y.^2 still has minior bugwith complex roots: highD
      % 'LangLocalGradSys'    : mu/4*(x^2-1)^2=mu/4*(x^4-2*x^2+1); 1D  double-well
      % 'LangLocal'           : overdamped system X(1,:)=position x; X(2,:)= momentum p; H=V(x)+K(p) 
      % 'Lorenz'              : three dim Lorenz system
      % 'EntropyBarrier2'     : second entropy barrier 
      % 'Langevin_x4sinx'     : damped Langevin system with pontial composed of local Liptchitz plus sin 
      % Langevin_MultiParticle: 
      
switch potential_type  
    case {'Ax2Bsin', 'Ax4Bsin'}
        [sysInfo,fmDataInfo]  = entropyBarrier(potential_type,scheme); 
    case {'x4','LangLocalGradSys'}   %  1D  double-well
        [sysInfo,fmDataInfo]  = potential1D(potential_type,scheme);  
    case 'GradDiss'
        [sysInfo,fmDataInfo]  = GradDiss_settings(potential_type,scheme); 
    case 'GradDissCouple'
        [sysInfo,fmDataInfo]  = GradDissCouple_settings(potential_type,scheme);
    case 'DoubleWell'
        [sysInfo,fmDataInfo]  = Double_well_settings(potential_type,scheme);
    case 'LangLocal'        
        [sysInfo,fmDataInfo]  = LangLocal_settings(potential_type,scheme);
    case 'Lorenz'
        [sysInfo,fmDataInfo]  = Lorenz_settings(potential_type,scheme);
    case 'EntropyBarrier2'
        [sysInfo,fmDataInfo]  = Entropy_Barrier2_settings(potential_type,scheme);
        
    case {'Langevin_x4sinx', 'Langevin_2well'}
        [sysInfo,fmDataInfo]  = Langevin_x1D_settings(potential_type,scheme);
        
   case 'Langevin_MultiParticle'   
        [sysInfo,fmDataInfo]  = Langevin_MultiParticle_settings(potential_type,scheme);
        
end

% we set fmDataInfo, the numerical setting, for each example, because it
% may depend on the example. 

    
%% set DIR for saving data
datapath           = [SAVE_DIR,potential_type, myslash,fmDataInfo.betaLambda,scheme, myslash];
if ~exist(datapath,'dir'), mkdir(datapath); end
datafileName       = [datapath,'Data_',fmDataInfo.type,'.mat'];
fmDataInfo.datapath     = datapath; 
fmDataInfo.datafileName = datafileName;  

end




