
function [lseSumInM,lseSumInM_array,lseAll_array,MseqL] = LSE_sumInMtraj(lseAb_all)
% Compute the MLE for different data size of Mseq = 2.^(10:16) trajectories
%{
Input
   lseAb_all  - a cell(M,1) of cells; lseAb_all{1,1} = lseAb_1traj; 
                but lseAb_1traj= cell(tseqL, gapseqL); 
                lseAb_1traj{1,1}.
                          Amat: [Npar×Npar×K double]
                          bvec: [Npar×K double]
                        res: [1xK]  
                          cvec: [Npar×K double]
OUTPUT
    lse_sumInM  - a cell(tseqL, gapseqL, MseqL)
                          Amat: [Npar×Npar×K]
                          bvec: [Npar×K double]
                        res: [1xK]  
                          cvec: [Npar×K double]
     lseSumInM_array  - a cell
                          Amat: [Npar×Npar×K,tseqL,gapseqL,MseqL]
                          bvec: [Npar×K,tseqL,gapseqL,MseqL]
                          cvec: [NparxK,tseqL,gapseqL,MseqL]
                        res: [1xK,tseqL,gapseqL,MseqL]  
    lseAll_cell3D - a cell(tseqL, gapseqL, M): M 1-traj-estimators
    lseAll_array  - a cell
                          Amat: [Npar×Npar×K,tseqL,gapseqL,M]
                          bvec: [Npar×K,tseqL,gapseqL,M]
                          cvec: [NparxK,tseqL,gapseqL,M]
                        res: [1xK,tseqL,gapseqL,M]  

     
regression solution: c = pinv(A)*b; 
res = \sum_n |y_n - c phi(x_n)|^2 =  alpha - 2 c'b + c'Ac = alpha - c'b;
%}
% M = trajNum
[M,~] = size(lseAb_all);   Mseq = [2.^(1:log2(M)), M];  MseqL = length(Mseq);     

lseAb_1traj = lseAb_all{1,1};   
[Npar,K ]   = size(lseAb_1traj{1,1}.cvec); 
[tL, gL]    = size(lseAb_1traj);

lseSumInM   = cell(tL, gL, MseqL);  % estimators from Mseq trajectories

% cell to array
% - it is easier to handle array than cell: can be computed/indexed easily  
% - but cell makes it easier to read the elements: coefs-sigma-
   % Amat_all   = zeros(Npar,Npar,K,tL,gL,M);   bvec_all       = zeros(Npar,K,tL,gL,M); 
cvec_all   = zeros(Npar,K,tL,gL,M);              
res_all = zeros(1,K,tL,gL,M);         
% sumInM array
Amat_SumInM   = zeros(Npar,Npar,K,tL,gL,MseqL);  bvec_SumInM       = zeros(Npar,K,tL,gL,MseqL); 
cvec_SumInM   = bvec_SumInM;                    
res_SumInM = zeros(1,K,tL,gL,MseqL);          

for tt = 1:tL
    for jj = 1:gL   
        AmatM   = zeros(Npar,Npar,K,M);     bvecM      = zeros(Npar,K,M); 
        cvecM   = bvecM;                    
        alphaM  = zeros(1,K,M);            
        resM = alphaM;                         
        MseqInd = 1; 
        for mm = 1:M
            lseAb_1traj          = lseAb_all{mm,1}; 
            temp                 = lseAb_1traj{tt,jj}.LSE_Abres; 
            
           % Get matrix array(Npar,Npar,K,M) to compute lseAb_sumInM
            A   = temp.Amat;  b  = temp.bvec; c = temp.cvec;   
            AmatM(:,:,:,mm)      = A;       bvecM(:,:,mm)       = b;  % (Npar,K) 
            cvecM(:,:,mm)        = c;       
            resM(:,:,mm)         = temp.res;  
            alphaM(:,:,mm)       = temp.res + sum(c.*b); 
            if MseqInd <= MseqL
                Mcurrent = Mseq(MseqInd);
                if mm == Mcurrent% compute lse_sumInM{tt,jj,MseqInd}
                    IndM   = 1:Mcurrent;
                    A0     = mean(AmatM(:,:,:,IndM),4);   b0       = mean(bvecM(:,:,IndM),3);
                    c0     = zeros(Npar,K);              
                    for kk=1:K    % update cvec
                        % c0(:,kk)     = A0(:,:,kk)\b0(:,kk);
                        c0(:,kk) = pinv(A0(:,:,kk))*b0(:,kk);
                    end
                    % update res
                    alpha0 = mean(alphaM(:,:,IndM),3);    
                    res0   = alpha0 - sum(c0.*b0);        
                    cell0.Amat   = A0; cell0.bvec = b0;   cell0.cvec = c0; 
                    cell0.res    = res0;                  
                    lseSumInM{tt,jj,MseqInd} = cell0;  % save to cell(tL,gL,MseqL)
                    % save to array
                    Amat_SumInM(:,:,:,tt,jj,MseqInd)   = A0; 
                    bvec_SumInM(:,:,tt,jj,MseqInd)     = b0;
                    cvec_SumInM(:,:,tt,jj,MseqInd)     = c0;         
                    res_SumInM(:,:,tt,jj,MseqInd)      = res0;
                    MseqInd = MseqInd+1;
                end
            end
        end
        % Get matrix array(Npar,Npar,K,L, gL, M) 
        % Amat_all(:,:,:,tt,jj,:)  = AmatM;   bvec_all(:,:,tt,jj,:)      = bvecM; 
        cvec_all(:,:,tt,jj,:) = cvecM;     % size (Npar,K,L, gL, M)
        res_all(:,:,tt,jj,:)  = resM; 
    end
end

% lseAll_array.Amat   = Amat_all;  lseAll_array.bvec = bvec_all;
lseAll_array.cvec   = cvec_all;      % size (Npar,K,L, gL, M)     Should not use squeez
lseAll_array.res    = res_all;   
lseAll_array.Mseq   = Mseq;

lseSumInM_array.Amat = Amat_SumInM; 
lseSumInM_array.bvec = bvec_SumInM;
lseSumInM_array.cvec = cvec_SumInM;       
lseSumInM_array.res  = res_SumInM;   
lseSumInM_array.Mseq = Mseq;
end
