
function analyze_estimator(lseSumInM,lseSumInM_array,gapseq,tseq,fmDataInfo,rmInfo, cvecAll_cell,gapInd_opt)
% analyze the estimators: 
%       - LSE matrix spectrum;  
%       - convergence in deltat= gap*dt
%       - convergence in Length of traj and number of traj; 
%{
    lse_sumInM  - a cell(tseqL, gapseqL, MseqL)
                          Amat: [Npar×Npar×K]
                          bvec: [Npar×K double]
                        resvec: [1xK]  
                          cvec: [Npar×K double]
     lseSumInM_array  - a cell
                          Amat: [Npar×Npar×K,tseqL,gapseqL,MseqL]
                          bvec: [Npar×K,tseqL,gapseqL,MseqL]
                          cvec: [NparxK,tseqL,gapseqL,MseqL]
                        resvec: [1xK,tseqL,gapseqL,MseqL]  
     cvecAll_cell - a 5 D double matrix [NparxK,tseqL,gapseqL,Mtotal] with Mtotal=1000;
%}


Mseq     = lseSumInM_array.Mseq;  L_Mseq = length(Mseq); 
L_gapseq = length(gapseq);        L_tseq = length(tseq); 

% analyze the LSE through Amat, cvec: 
gapInd = L_gapseq-2;  
LSE_Abres.Amat       = lseSumInM_array.Amat(:,:,1,end,gapInd,end);
LSE_Abres.bvec       = lseSumInM_array.bvec(:,1,end,gapInd,end);
LSE_Abres.cvec       = lseSumInM_array.cvec(:,1,end,gapInd,end);
LSE_Abres.res        = lseSumInM_array.res(:,1,end,gapInd,end);

rm_string = rmInfo.nameStrFn(gapseq(gapInd));  
figname0  = ['fig_eigA',rm_string];  figpath   = rmInfo.fig_path; 
printON   = 0; 
if printON==1; LSE_analyze(LSE_Abres,printON,figname0,figpath); end

%% Part I: convergence of coefficients and residual in delta=Gap*dt, using the larges sample size
estimatorSeq = lseSumInM(L_tseq,1:L_gapseq,L_Mseq);  %  cell(tseqL, gapseqL, MseqL);

%% pass cvecAll to plot function to compute std of each cvec

arraySize = size(cvecAll_cell);    % make cvecAll to be 5D : (Npar,K,1, gL, M)
M         = arraySize(end);
if length(arraySize) == 5
    cvecAll = cvecAll_cell(:,:,L_tseq,1:L_gapseq,:);   % size (Npar,K,L, gL, M)  >> size (cvecAll)  =   (Npar,K,1, gL, M)
else
    K=1; Npar = length(LSE_Abres.cvec);
    cvecAll   = zeros(Npar,K,1,L_gapseq,M);  
    cvecAll(:,K,:,:,:) = cvecAll_cell(:,L_tseq,1:L_gapseq,:);     % size (Npar,L, gL, M) due to the previous use of squeeze in LSE_sumInMtraj
end

plot_Conv_deltat(gapseq,estimatorSeq,fmDataInfo,rmInfo,cvecAll,gapInd_opt);

%% Part II: Convergence in Mseq=[traj-Length and M-traj] (Ergodicity related) at the optimal
% time gap gapInd_opt when TVD is smallest
%%%% >>> done outside of this function 

%{
% plot_Conv_Mtraj(); 
tseqInd = 1; % from 1:L_tseq
estimatorSeq = lseSumInM(tseqInd,gapInd_opt,:);  %  cell(tseqL, gapseqL, MseqL);

%% pass cvecAll to plot function to compute std of each cvec

arraySize = size(cvecAll_cell);    % make cvecAll to be 5D : (Npar,K,1, gL, M)
M         = arraySize(end);
if length(arraySize) == 5
    cvecAll = cvecAll_cell(:,:,L_tseq,gapInd_opt,:);   % size (Npar,K,L, gL, M)  >> size (cvecAll)  =   (Npar,K,1, gL, M)
else
    K=1; Npar = length(LSE_Abres.cvec);
    cvecAll   = zeros(Npar,K,1,L_gapseq,M);  
    cvecAll(:,K,:,:,:) = cvecAll_cell(:,L_tseq,gapInd_opt,:);     % size (Npar,L, gL, M) due to the previous use of squeeze in LSE_sumInMtraj
end

plot_Conv_Mtraj(Mseq,estimatorSeq,fmDataInfo,rmInfo,cvecAll,gapInd_opt);
%}

end
    





