function [cvec,res,LSE_Abres]= estimatePara_MLE(X, dWDeltat,rmInfo,sysInfo,showRank)
% estimate the parameters by MLE
%{
settings of the scheme to be inferred: 
Euler form     Y(n+1) = Yn - deltat * termsEuler(Yn,deltat) + sigma1*sqrt(deltat)*Bn 
               termsEuler(Yn,deltat) =  [c0*Yn - c1* gradV(Yn)]    
IT1p5 form     Y(n+1) = Yn - deltat * termsIT1p5(Yn,deltat) + simga1*deltat*HessV(Yn)*Wn       
               termsIT1p5(Yn,deltat,Bn) = [c0*Yn - c1* gradV(Yn) + c2 LgV(Yn)*deltat + ce ]   
data_fm = struct( ...
    'X',        X,...
    'dWdt',     dWdt, ...
    'tN',       tN, ...
    'dt',       dt, ...
);
rmInfo     = struct(...     
    'rm_type',   scheme_type,...
    'obsgap',    obsgap,...
    'deltat',    deltat,...
    'tLagIndex', tLagIndex, ...
    'rm_path',   [datapath, scheme_type], ...
    'figname0',  figname0,...
    'estimator_filename',  estimator_filename ...
    );
%}

if ~exist('showRank','var');  showRank  = 0; end

%% estimate the parameters 
Xn       = X(:,1:end-1);   XnDiff = ( X(:,2:end)-Xn )/rmInfo.deltat;  

switch rmInfo.rm_type
    case 'IT1p5'
        deltat = rmInfo.deltat; useNoise = rmInfo.useNoise; 
        beta   = sysInfo.beta; gradV    = sysInfo.gradV; HessV = sysInfo.hessV; 
        LaplaceGradV     = sysInfo.LaplaceGradV; 
        [terms,MatHessV] = termsIT1p5(Xn,gradV,HessV,LaplaceGradV,deltat,useNoise,dWDeltat,beta);
        [XnDiff,terms]   = IT1p5_trans2LSE(XnDiff,terms, MatHessV,useNoise); 
    otherwise
        termsFn = rmInfo.termsFn;
        terms   = termsFn(Xn,rmInfo.deltat,rmInfo.useNoise,dWDeltat);
end

if rmInfo.termYnOFF == 0   % If YnOFF==0, use  term c0*Yn
    [cvec,res,LSE_Abres] = LSE_Axb(XnDiff,terms,showRank,rmInfo.lse_entrywise);
else                       % otherwise, remove c0*Yn
    [Npar,~] = size(terms);   % number of parameters
    terms = terms(2:Npar); 
    [cvec,res,LSE_Abres] = LSE_Axb(XnDiff,terms,showRank,rmInfo.lse_entrywise);
end

end

