%% inferScheme -- infer a numerical scheme with a larger time step size
%   + as fast as an explicit scheme, an also get invariant measure as Implicit scheme
%{
Algorithm: 
   + generate data from an Implicit scheme (or another that accurately approximates the invariant measure)
   + estimate the parameters; 
   + run the infered scheme 
% Last updated 2020/12/29;  started 2020/12/23 
%} 

%% change the settign and run this code 
settings_inferScheme;    


%% downsample data
[X, dWDeltat] = DownSample_Data(data_fm,rmInfo); % downsample data ; dWDeltat ~ sqrt(deltat)*Bn 

[cvec,res,LSE_Abres] = estimatePara_MLE(X, dWDeltat,rmInfo); 
fprintf('Estimated para (should be [0,1] for Euler):\n'); disp(cvec'); 
fprintf('Residual scaled by *deltat %2.4f (var(noise)= %2.4f) \n',res*rmInfo.deltat, (sysInfo.sigma_1)^2); 


%% run the inferred system
x0 = X(:,1); tN = length(X(1,:))-1; deltat = rmInfo.deltat; 
useNoise = rmInfo.useNoise; 
termsFn  = rmInfo.termsFn; 
Xtraj = run_inferred_scheme(cvec,res,termsFn,tN,x0,deltat,useNoise,dWDeltat); 

tInd = (1:1:tN+1); tt = deltat*tInd; 
plot(tt,Xtraj(1,tInd),'b-', tt, X(1,tInd),'r:'); xlabel('Time t (in units)')
legend('Reproduced','True');


%% To test multiple steps later
%{
%% set Traj length sequence, and observation gap sequence 
TrajNum  = 1;                           % number of trajectories
if TrajNum ==1;      trajL_seq = 10*[2,4,8,16,32,64,128,256];  % length of trajectory
else;                trajL_seq = 10*[2,4,8,16,32,64,128];  % length of trajectory 
end
fmDataInfo.obsgap   = 10;  
if fmDataInfo.obsgap == 10;     obsGap_seq  = fmDataInfo.obsgap* [1,2,4,6,8,10];  % observation gap for RM: larger gaps
elseif fmDataInfo.obsgap == 1;  obsGap_seq  = fmDataInfo.obsgap*( 5*(1:6));       % observation gap for RM: small gaps
end
rmInfo.trajL_seq   = trajL_seq; 
rmInfo.obsGap_seq  = obsGap_seq; 

noiseON     = rmInfo.noiseON;     % observation noise 

%% Filenames
settingsType = [sprintf('Re%iKK%iK%i',Re,parOM.NoiseKK,rmInfo.K),noiseStd,sprintf('_Gap%i',rmInfo.obsgap)]; 
mypath       = [mypath,settingsType,'/']; if ~exist(mypath,'dir'); mkdir(mypath); end
obsData      = [mypath,'obsData_', settingsType,'.mat'];   % for single traj inference

timeL = 1;    % timeL <=1; 
switch timeL
    case 1;        type_T='';      case 0.5;     type_T='_T1000';
    case 0.25;    type_T='_T500';  case 0.125;   type_T= '_T250'; 
end


NARtype  = [sprintf('L%i',rmInfo.wlag), rmInfo.vFNLagType, type_T,sprintf('_M%i',TrajNum)]; 
fileMLE  = [mypath,NARtype,'.mat'];   % estimator file


%% Generate initial conditions  
[uICs,spTrue]  = generateICs(parOM, ICpath); % generate/load IC samples of invariant mealsure from a long trajectory;
[~,nICs] = size(uICs);

%% Estimate parameters if the file does not exist: estimate para for Mseq, TseqL
if exist(fileMLE,'file') == 0    
    %% estimate parameters  
%     if TrajNum==1; u0 = uICs(:,1);         % a single trajectory demostration:
%        inferFrom1traj_LSE(u0,parOM,rmInfo,noiseON, 1,fileMLE);
%     else                 % many trajectories and different lengths of trajectories
        if TrajNum > nICs; fprintf('Traj-number should be < %i\n', nICs); end
        inferFrom_Mtraj(uICs(:,1:TrajNum),parOM,rmInfo,noiseON,fileMLE);
%    end
end

% p = gcp('nocreate'); % If no pool, do not create new one.
% poolsize = p.NumWorkers; 
% if poolsize >10; return; end   % if running on cluster, stop here. 

%% load and run reduced model
load(fileMLE);                        % plot the LSE estimators
[M,N]     = size(lseAb_all);       M = 1;
% plot_MTgap_2;
gapInd   = 2;  gap = rmInfo.obsGap_seq(gapInd);
fprintf('\n Inference from M = %i trajectories with T = %i. Gap=%i\n', M, trajL_seq(end),gap);
if M ==1                              % estimator from 1traj
    cc        = lseAb_all{1,1};       % load LSE_Abres from the cell array
    LSE_Abres = cc{end,gapInd};       %  a cell(tseqL, gapseqL)
else  % Many trajectory data processing
    if ~exist('lseSumInM_array','var')    % compute LSE for different number of trajectories
        [lseSumInM,lseSumInM_array,lseAll_cell3D,lseAll_array] = LSE_sumInMtraj(lseAb_all);
        save(fileMLE,'lseSumInM_array','-append');
    end
    plotON = 0;
    if plotON==1; plot_MTgap_2; end   % plot the residual and estimator change in T, M, gap
    
    LSE_Abres.Amat       = lseSumInM_array.Amat(:,:,:,end,gapInd,end);
    LSE_Abres.bvec       = lseSumInM_array.bvec(:,:,end,gapInd,end);
    LSE_Abres.cvec       = lseSumInM_array.cvec(:,:,end,gapInd,end);
    LSE_Abres.cvecPinv   = lseSumInM_array.cvecPinv(:,:,end,gapInd,end);
    LSE_Abres.resvec     = lseSumInM_array.resvec(:,:,end,gapInd,end);
    LSE_Abres.resvecPinv = lseSumInM_array.resvecPinv(:,:,end,gapInd,end);
end
NARtype  = [sprintf('L%i',rmInfo.wlag), rmInfo.vFNLagType, type_T,sprintf('_M%i',M)];

sparsifyLASSO = 0;  % sparcify the coefficients. 0 for none; 1 for Sequential thresholding; 2 for LASSO
printON  = 1;  % print the LSE plots
if sparsifyLASSO ==1
    hard_threshold = 0.001; 
    if      hard_threshold ==0.001;  NARtype = [NARtype, 'sparse001'];
    elseif  hard_threshold ==0.0001; NARtype = [NARtype, 'sparse0001'];
    end
    if ~isfield(LSE_Abres,'cvec_sparse')
        [~,~,LSE_Abres] = LSE_sparcify(LSE_Abres,hard_threshold); % This step is easy and fast, not saved. 
    end
    fig_lse_name = [NARtype,'_gap',num2str(gap)];
    LSE_analyzeSparse(LSE_Abres,printON,fig_lse_name,mypath);
elseif sparsifyLASSO ==2 
    NARtype = [NARtype, 'lasso']; 
    if ~isfield(LSE_Abres,'cvec_lasso')
        [cvec_lasso,res_lasso,LSE_Abres] = LSE_lasso(LSE_Abres); % This step is easy and fast, not saved. 
    end
    fig_lse_name = [NARtype,'_gap',num2str(gap)];
    LSE_analyzeLASSO(LSE_Abres,printON,fig_lse_name,mypath);
    
elseif sparsifyLASSO == 0
    fig_lse_name = [NARtype,'_gap',num2str(gap)];
    LSE_analyze(LSE_Abres,printON,fig_lse_name,mypath);
end 

fprintf('\n Estimator from M = %i trajectories with T = %i. Gap=%i, L=%i, Sparcify =%i \n', ...
             M, trajL_seq(end),gap,rmInfo.wlag,sparsifyLASSO);
run_RM(NARtype,gap,mypath,LSE_Abres,rmInfo,parOM,ICpath,sparsifyLASSO);     % run the reduced model

%}



