 % function inferScheme_Mtraj(ICs,sysInfo,fmDataInfo,rmInfo,Mtraj,gapseq)
% inference using many trajectories (= long trajectory)
% - can computed in parallel: each inferS_1traj returns an lseAb, then assemble here
% - test consistency:   in obsgap, in time, in Mtraj
% - present results: 
%    + convergence of estimator
%    + 1traj comparision

%% Indicator for initial condition generator 
global IC_inv
% IC_inv=0; %% for fixed initial condition
% IC_inv=1; %% for initial condition downsample (generated) from the
% invariant measure

%% use a smaller length for M-trajecotaries
tN0   = ceil(fmDataInfo.tN/Mtraj);
tseq  = tN0*2.^(1:1:(log2(Mtraj)-2));   % a sequence of lengths for trajectory to test converngece in time to the invariant measure
if strcmp(sysInfo.potential_type,'Lorenz'); tseq  = tN0*2.^(1:1:(log2(Mtraj)-1));end 
fmDataInfo1 = fmDataInfo; fmDataInfo1.tN = tseq(end);

%% filenames
rm_string     = rmInfo.nameStrFn(gapseq(end)); 
if ~contains(rm_string,'_M')   % update due to filename change
    rmInfo.nameStrFn = @(n)[rmInfo.nameStrFn(n), sprintf('_M%i',Mtraj)];   % add _M# to all the file names   
    rm_string     = rmInfo.nameStrFn(gapseq(end)); 
end

gapStr        = strrep(rm_string,'Gap',sprintf('Gap%ito',gapseq(1)));    
Est_filename  = [rmInfo.rm_path, 'Estimator',gapStr,'.mat'];
if rmInfo.lse_entrywise == 1
    Est_filename = [rmInfo.rm_path, 'Estimator',gapStr,'_lseEntrywise.mat'];
end

if IC_inv==0
    Est_filename = [rmInfo.rm_path, 'Estimator',gapStr,'_FixedIC.mat'];
end

filenameChange(rmInfo,Est_filename,gapseq,Mtraj)
rmInfo.gapStr = gapStr;                 rmInfo.Est_filename = Est_filename;

%% Estimate the coefficients by LSE: in for each tseq x gapseq; parallel in M
fprintf('Infererence from M=%i trajectories: \n', Mtraj);
if ~exist(Est_filename,'file')
    tic;     [~,Mtraj] = size(ICs);
    lseAb_all = cell(Mtraj,1);     fprintf('\n  Progress: \n ');   
    if Mtraj ==1
        x0 = ICs(:,1);
        lseAb_1traj  = inferS_1traj(x0,rmInfo,sysInfo,fmDataInfo,gapseq,tseq,1,1);   
        lseAb_all{1} = lseAb_1traj;    % lseAb_1traj. Amat: [Npar×Npar×K double]    bvec: [Npar×K double]
                                       %               res: [1xK]                   cvec: [Npar×K double]
    else
        parfor nn = 1:Mtraj
            x0 = ICs(:,nn);
            lseAb_1traj   = inferS_1traj(x0,rmInfo,sysInfo,fmDataInfo1,gapseq,tseq,0,0);
            lseAb_all{nn} = lseAb_1traj;  
            fprintf(' nn = %i out of %i \n',nn, Mtraj);
        end
    end
    TimeTaken = toc; 
    [lseSumInM,lseSumInM_array,lseAll_array,Mseq] = LSE_sumInMtraj(lseAb_all);   % assemble (compute) the estimators; 
    
    save data.mat  lseAb_all sysInfo rmInfo fmDataInfo Mtraj gapseq tseq lseSumInM lseSumInM_array lseAll_array TimeTaken Mseq; 
    movefile('data.mat', Est_filename); 
end
load(Est_filename);  
% update due to filename change
rm_string     = rmInfo.nameStrFn(gapseq(end));  
rmInfo.gapStr = gapStr;
if ~contains(rm_string,'_M')
    rmInfo.nameStrFn = @(n)[rmInfo.nameStrFn(n), sprintf('_M%i',Mtraj)];   % add _M# to all the file names
    save(Est_filename,'rmInfo','-append');
end

%%%--------------------------------------------
%% Present results:
%% 1. compare the PDF and ACF from 1 trajectory
if ~exist(fmDataInfo.datafileName,'file')      % run the full model 
    X_init        = fmDataInfo.getInitial(1);
    data_fm       = generateData(sysInfo,fmDataInfo, 0,1,X_init);            % (sysInfo,fmDataInfo,plotON,saveON,X_init)
else 
    load(fmDataInfo.datafileName,'data_fm'); 
end 
load(Est_filename); 

plotPDF_ACF = 0; 
gapStartInd = 4;
if ~exist('TV_RMvsTrue_array','var') || plotPDF_ACF == 1
    % downsample and compare with the RM:
    Lgapseq = length(gapseq);
    rel_err = zeros(1,Lgapseq); rel_err_Euler = rel_err;
    
    %% compute the total variation distance between RM measure and true measure;
    %% RM0 measure and true measue, respectively verse each time gap
    TV_RMvsTrue_array=zeros(1,Lgapseq-gapStartInd+1);
    TV_RM0vsTrue_array=TV_RMvsTrue_array;

    for gapInd = (gapStartInd:1:10)
        % gapInd        = 8; % length(gapseq)-1;          % select the gapInd here: 4-10    gapseq = [1, 2, 4, 10, 20,   40, 80, 120, 160,200];% ,
        obsgap        = gapseq(gapInd);            rmInfo.obsgap = obsgap;
        [X, dWDeltat] = DownSample_Data(data_fm,rmInfo);                  % downsample data ; dWDeltat ~ sqrt(deltat)*Bn
        cvec          = lseSumInM_array.cvec(:,:,end,gapInd,end);         % size = (Npar,K,tL,gL,MseqL)
        res           = lseSumInM_array.res(:,:,end,gapInd,end);          % size = (1,K,tL,gL,MseqL)
        [rel_err_var,TV_RMvsTrue, TV_RM0vsTrue,scheme_cmp,blowupFlag,blowupEuler]  = compare_1traj(X,dWDeltat,cvec,res,rmInfo,sysInfo,fmDataInfo,gapInd);     % run the RM, and compare with FM and Euler
        rel_err(gapInd)       = rel_err_var.InferredScheme;
        rel_err_Euler(gapInd) = rel_err_var.Euler;
        %   store the total variation distances
        if blowupFlag;     TV_RMvsTrue=1;         end      
        if blowupEuler;    TV_RM0vsTrue=1;         end
        
        TV_RMvsTrue_array(gapInd-gapStartInd+1)=TV_RMvsTrue;
        TV_RM0vsTrue_array(gapInd-gapStartInd+1)=TV_RM0vsTrue;     
    end
    save(Est_filename,'rmInfo','rel_err_Euler','rel_err','TV_RMvsTrue_array','TV_RM0vsTrue_array','scheme_cmp','gapStartInd','-append');
end
plot_TVD_relErr(scheme_cmp,fmDataInfo,gapStartInd,TV_RMvsTrue_array,TV_RM0vsTrue_array,rmInfo,gapseq,rel_err,rel_err_Euler);



%% 2. convengence of estimator:  in obsgap, in time, in M
%%% Part I: Plot convergence of coefficients vs number of traj M stored in Mseq
[~,gapInd_temp]=min(TV_RMvsTrue_array); gapInd_opt=gapInd_temp+gapStartInd-1; 
% Mseq = [2.^(1:log2(Mtraj)), Mtaj]; 
% pass LseAll_array.cvec into the function to compute std of vec
analyze_estimator(lseSumInM,lseSumInM_array,gapseq,tseq,fmDataInfo,rmInfo,lseAll_array.cvec, gapInd_opt); % analyze estimator: LSE, convengence in obsgap, in time, in M

%%% Part II: Convergence in Mseq=[traj-Length and M-traj] (Ergodicity related) at the optimal
% time gap gapInd_opt when TVD is smallest
paraIndex = 2-rmInfo.termYnOFF; 
Mseq      = lseAll_array.Mseq; 
plot_Conv_MT(lseSumInM,gapInd_opt,paraIndex,tseq,Mseq,rmInfo);
