function [terms,MatHessV] = termsIT1p5(Yn,gradV,HessV,LaplaceGradV,deltat,useNoise,Bn,beta)
% the terms in scheme Ito-Taylor 1.5 for inference 
%{
IT1.5 form     Y(n+1) = Yn - deltat * termsIt1p5(Yn,deltat) + sqrt(res)*MatHessV*N(0,1)
      1. (useNoise=1) with dBn: 
                terms = [Yn, (I-deltat/2)HessV*gradV, deltat*LaplaceGradV, [1-sqrt(deltat)/2*HessV]*dBn] 
          with multiplicative noise:  sqrt(deltat)*HessV*eta
                MatHessV = sqrt(deltat)*HessV
      2.  (useNoise=0) without dBn, but keep O(delta^1.5): 
                terms = [Yn, (I-deltat/2)HessV*gradV, deltat*LaplaceGradV]
           with multiplicative noise: 
                MatHessV =  sqrt(H1^2+H2^2)
            where  H1 = [1-sqrt(deltat)/2*HessV]/sqrt{delta},    % here use /sqrt(dt) because dBt ~ sqrt(dt)
                   H2 = sqrt(delta/12)*HessV;  
       3. (useNoise=11) with dBn, drop O(delta^1.5):  
                terms = [Yn, (I-deltat/2)HessV*gradV, deltat*LaplaceGradV, [1-sqrt(deltat)/2*HessV]*dBn] 
          with additive noise:  res  
              MatHessV =  Id
       4. (useNoise=10) without dBn, drop O(delta^1.5): 
                terms = [Yn, (I-deltat/2)HessV*gradV, deltat*LaplaceGradV]
           with multiplicative noise: [1-sqrt(deltat)/2*HessV]*dBn
              MatHessV = [1-sqrt(deltat)/2*HessV]
%}

[dimX,tN] = size(Yn); 
I_d       = eye(dimX);
delta_c4  = sqrt(deltat/(6*beta));     % delta^{1/2}
sqrt_beta = sqrt(2/beta); 
delta_c2  = deltat/(2*beta);           % delta/(2*beta)
sqrt_delta  = sqrt(deltat)/2; 
t2 = zeros(dimX,tN); t3=t2; t4=t2;  MatHessV = zeros(dimX,dimX,tN);

%% compute the terms trajectory, and the multiplicative matrix in front of the new noise term
for t=1:tN
    ynt       = Yn(:,t); 
    HessV_val = HessV(ynt);
    gradV_val = gradV(ynt);
    t2(:,t)    = - (I_d- deltat/2*HessV_val)*gradV_val;   % term with c1  --- c0 is for Yn
    t3(:,t)   = -delta_c2 * LaplaceGradV(ynt);           % term with c2
    
    H1        = (I_d - sqrt_delta* HessV_val )*sqrt_beta;  % H1 * dBn
    switch useNoise     % the matrix in the 4th term or the multiplicative noise
        case 0          % without dBn, but keep O(delta^1.5): 
            H2              = HessV_val*delta_c4;
            MatHessV(:,:,t) = sqrtm(H1'*H1/deltat + H2'*H2);
        case 1          % with dBn (=>> t4). H2 used in case=1;
            t4(:,t)         = H1 * Bn(:,t);
            H2              = HessV_val*delta_c4;  
            MatHessV(:,:,t) = H2;            
        case 10         % without dBn, drop O(delta^1.5): 
            MatHessV(:,:,t) = H1/sqrt(deltat);   
        case 11         % with dBn, drop O(delta^1.5): 
           t4(:,t)         = H1 * Bn(:,t); 
           MatHessV(:,:,t) = I_d; 
    end
end

switch useNoise
    case {0,10}
        terms = cell(3,1);
        terms{1} = Yn;    terms{2} = t2;   terms{3} = t3; 
    case {1,11}
        terms = cell(4,1);
        terms{1} = Yn;    terms{2} = t2;   terms{3} = t3;  terms{4} = t4;
end
end
