function plot_Conv_MT(lseSumInM,gapInd_opt,paraIndex,tseq,Mseq,rmInfo)
% plot converngence of the estimator in MT: M=# trajectories; T = traj length
%{
Input
 lseSumInM: cell(tseqL, Optimal_gap, MseqL);  size = (tseqL, 1, MseqL)
    aa = lseSumInM{1,1,1}   ==>>>
    aa =   struct with fields:
        Amat: [Npar×NparxK double]
        bvec: [Npar×K double]
        cvec: [Npar×K double]
        res: [1,K]
Output
  Plot: relative errors v.s. MT;    all points as dots, colored in MseqL
%}
cvec_MT = lseSumInM{end,gapInd_opt,end}.cvec;

% Select tseqInd and mseqInd to plot: the last one not included
tseqInd = 1:1:length(tseq)-1;    tseq1 = tseq(tseqInd); 
mseqInd = 2:2:length(Mseq)-1;    mseq1 = Mseq(mseqInd);
estimatorSeq = lseSumInM(tseqInd,gapInd_opt,mseqInd);  %  cell(tseqL, gapseqL, MseqL);

[tseqL, gapL, MseqL] = size(estimatorSeq); 
if gapL >1; fprintf('Select the optimal gap in gapseq!\n'); end
 
[Npar,K] = size(cvec_MT);
cvecAll = zeros(Npar,K,tseqL,MseqL);
relerrAll  = zeros(Npar,K,tseqL,MseqL);
tm_product = zeros(tseqL, MseqL);
for tt = 1:tseqL-1
    for mm = 1:MseqL
       cvecAll(:,:,tt,mm)    = estimatorSeq{tt,gapL,mm}.cvec;
       relerrAll (:,:,tt,mm) = abs( (cvecAll(:,:,tt,mm)- cvec_MT)./cvec_MT);
       tm_product(tt,mm)     = tseq1(tt)*mseq1(mm);
    end
end

figure; 
colorMseq = colormap(parula(MseqL)); 
for k=1:K
    logTM_all = []; logErr_all = []; 
    ax(k) = subplot(K,1,k); 
    for mm = 1:MseqL 
        logTM  = log10(tm_product(:,mm)); 
        logErr = log10(squeeze(relerrAll(paraIndex,k,:,mm))); 
        if mm==1; plot(logTM(1), logErr(1),'o','MarkerSize',6, 'MarkerFaceColor',colorMseq(mm,:)); hold on; end
        plot(logTM, logErr,'o','MarkerSize',6, 'MarkerFaceColor',colorMseq(mm,:),'HandleVisibility','off'); hold on;
        logTM_all  = [logTM_all, logTM]; logErr_all = [logErr_all,logErr]; 
       if k==K; xlabel('log_{10}(N\times M)');  end
       ylabel('log_{10}(Rel. err)');
    end
    ind_Inf = isinf(logErr_all);
    [a,~]   = polyfit(logTM_all(~ind_Inf),logErr_all(~ind_Inf),1);
    y       =  -0.5*(logTM_all-logTM_all(2)) + logErr_all(1);
    plot(logTM_all, y,'k','linewidth',1);
    legend('Rel. error','Slope = -0.5','Location','SouthWest');
end


xtickM = cell(MseqL*2+1,1);
for mm=1:MseqL
    xtickM(1+(mm-1)*2) = {''};
    xtickM(mm*2) = {sprintf('%i',mseq1(mm))};
end
xtickM(MseqL*2+1) = {''};

 cb =colorbar('XTickLabel',xtickM);   % set the colorbar tick and label. 
% cb =colorbar;
set(cb, 'Position', [.80 .11 .04 .76])  % [x0 y0 width height]
for i=1:K
      pos=get(ax(i), 'Position');
      set(ax(i), 'Position', [pos(1) pos(2) 0.86*pos(3) pos(4)]);
end

colorTitleHandle = get(cb,'Title');  titleString = 'M'; set(colorTitleHandle ,'String',titleString);
 set(cb,'Ticks',0:0.125:1, 'XTickLabel',xtickM);   %% problem here. 
% set(cb,'XTickLabel',xtickM);


    set_positionFonts;
    % tightfig;
    figname = [rmInfo.fig_path, 'conv_TM_',rmInfo.gapStr];

    savefig(gcf,[figname,'.fig']); 
    set(gcf,'Position',  [100, 1000, 500, 300+100*K]); % tightfig;
    % set(gcf,'Position',  [100, 1000, 500, 400]); % [x0,y0, width, height]
    print([figname,'.eps'],'-depsc');
    
end


