
function plot_Conv_deltat(gapseq,estimatorSeq,fmDataInfo,rmInfo,cvecAll,gapInd_opt)
% plot the convergence of coefs&res in deltat= gap*dt; 

[L_cvec,K]  = size(estimatorSeq{1}.cvec);

normalizeBYdeltat = 0; 
if K==1
   plot_LSE_nonEntrywise(gapseq,estimatorSeq,fmDataInfo,rmInfo,cvecAll,normalizeBYdeltat,gapInd_opt);
elseif K>1
   plot_LSE_Entrywise(gapseq,estimatorSeq,fmDataInfo,rmInfo,cvecAll,normalizeBYdeltat,gapInd_opt);
end
end


function plot_LSE_nonEntrywise(gapseq,estimatorSeq,fmDataInfo,rmInfo,cvecAll,normalizeBYdeltat,gapInd_opt)
    L_gapseq    = length(gapseq);
    [L_cvec,~]  = size(estimatorSeq{1}.cvec);
    cvec_array  = zeros(L_gapseq, L_cvec);
    res_array   = zeros(L_gapseq, 1);
    delta_array = fmDataInfo.dt*gapseq;
    termYnOFF   = rmInfo.termYnOFF;
    
%%  store the standard derivation std error of each coefficient for each given gapseq with respect to M=1000 traj
err=zeros(L_gapseq,L_cvec);k=1;    
    for ll=1:L_gapseq
        deltat = delta_array(ll);    % obsgap = gapseq(ll); % should use deltat = obsgap *dt instead of obsgap
        if normalizeBYdeltat ==0; deltat = 1; end      % remove deletat normalizing, to match the new draft
        for j=1:L_cvec
            cvec_array(ll,j)=  deltat* estimatorSeq{ll}.cvec(j);   
            %% compute the errorbar for the estimators;
            err(ll,j)=std((cvecAll(j,k,1,ll,:)-cvec_array(ll,j)));
        end
        res_array(ll)=  deltat* sqrt(estimatorSeq{ll}.res);  
    end
    
    markerseq ={'-o','-x','-.>','-.+','-d-'};
    figure;
    subplot(121);  lgnd = cell(L_cvec,1);
    for j=1:L_cvec
        % plot(delta_array, cvec_array(:,j),markerseq{j},'Linewidth',1);hold on    
        %% Plot errorbar for the estimators;
        errorbar(delta_array, cvec_array(:,j),err(:,j),markerseq{j},'Linewidth',1); hold on; 
        x       = delta_array; y = cvec_array(:,j);
        [a,~]   = polyfit(x,y,1);
        %% when termYnOFF==0, legend starts from c_0
        if termYnOFF
            lgnd{j} = [sprintf('c_%i = %2.2f ',j,a(1)),'\delta',sprintf('+ %2.2f',a(2))];     lgnd{j} = sprintf('c_%i ',j); 
        else
            lgnd{j} = [sprintf('c_%i = %2.2f ',(j-1),a(1)),'\delta',sprintf('+ %2.2f',a(2))]; lgnd{j} = sprintf('c_%i ',j-1); 
        end
    end
    legend(lgnd,'Location','best'); xlabel('\delta'); ylabel('Coefficients');  hold off
     h1 = xline(delta_array(gapInd_opt),'--','Linewidth',1.5);   % add a vertical line at the optimal time gap
     set(get(get(h1,'Annotation'),'LegendInformation'),'IconDisplayStyle','off');
    
    
    subplot(122);
    loglog(delta_array, res_array,'-v','Linewidth',1); hold on;        % %% TODO: errorbar for the estimators; NOTE the loglog effect --- how to do it?
    ind = 3:L_gapseq-1;     % regression using only the middle points
    x     = log(delta_array); y = log(res_array);
    [a,~] = polyfit(x(ind),y(ind),1);     xlabel('\delta');  
    if normalizeBYdeltat == 1    % remove deletat normalizing, to match the new draft
        ylabel('\delta\sigma_\eta ');
    lgnd  = ['log (\delta\sigma_\eta) ', sprintf('= %2.2f', a(1)),'log(\delta)',sprintf('+ %2.2f', a(2))];
    else
        ylabel('\sigma_\eta ');
        lgnd  = ['log (\sigma_\eta) ', sprintf('= %2.2f', a(1)),'log(\delta)',sprintf('+ %2.2f', a(2))];
    end
    y_ref       =  0.5*(x(ind)-x(ind(1))) + y(ind(1));   loglog(delta_array(ind), exp(y_ref),'k--','linewidth',1);   % the plot is log-log, 
    lgnd  = {lgnd,'Reference slope = 0.5'};
   
    legend(lgnd,'Location','best');
    
    
set_positionFonts;
figname = [rmInfo.fig_path, 'conv_',rmInfo.gapStr];
savefig(gcf,[figname,'.fig']); 
set(gcf,'Position',  [100, 1000, 800, 400]);tightfig;
print([figname,'.pdf'],'-dpdf');
print([figname,'.eps'],'-depsc');
end


function plot_LSE_Entrywise(gapseq,estimatorSeq,fmDataInfo,rmInfo,cvecAll,normalizeBYdeltat,gapInd_opt)
L_gapseq    = length(gapseq);
[L_cvec,K]  = size(estimatorSeq{1}.cvec);   % K = dimX; L_cvec = dim parameter
 
figure;
for k=1:K
    cvec_array  = zeros(L_gapseq, L_cvec);
    res_array   = zeros(L_gapseq, 1);
    delta_array = fmDataInfo.dt*gapseq;
    termYnOFF   = rmInfo.termYnOFF;
    
    err=zeros(L_gapseq,L_cvec); % store std for each coefficient at each given gapseq
    for ll=1:L_gapseq
        deltat = delta_array(ll);    % obsgap = gapseq(ll); % should use deltat = obsgap *dt instead of obsgap
        if normalizeBYdeltat ==0; deltat = 1; end      % remove deletat normalizing, to match the new draft
        for j=1:L_cvec
            cvec_array(ll,j)=  deltat* estimatorSeq{ll}.cvec(j,k);
            %% compute the errorbar for the estimators;
            err(ll,j)=std((cvecAll(j,k,1,ll,:)-cvec_array(ll,j)));
        end
        res_array(ll)=  deltat* sqrt(estimatorSeq{ll}.res(k));
    end
    
    markerseq ={'-o','-x','-.>','-.+','-d-'};
    subplot(K,2,1+(k-1)*2);  lgnd = cell(L_cvec,1);
    for j=1:L_cvec
        plot(delta_array, cvec_array(:,j),markerseq{j},'Linewidth',1);hold on    
        %% plot errorbar for the estimators;
        errorbar(delta_array, cvec_array(:,j),err(:,j)); 
        
        h1 = xline(delta_array(gapInd_opt),'--','Linewidth',1.5);   % add a vertical line at the optimal time gap
        set(get(get(h1,'Annotation'),'LegendInformation'),'IconDisplayStyle','off');
        %  if normalizeBYdeltat ==1   % remove deletat normalizing, to match the new draft
        x       = delta_array; y = cvec_array(:,j);
        [a,~]   = polyfit(x,y,1);
        %% when termYnOFF==0, legend starts from c_0
        if termYnOFF
            lgnd{j} = [sprintf('c_%i = %2.2f ',j,a(1)),'\delta',sprintf('+ %2.2f',a(2))];  lgnd{j} = sprintf('c_%i ',j); 
        else
            lgnd{j} = [sprintf('c_%i = %2.2f ',(j-1),a(1)),'\delta',sprintf('+ %2.2f',a(2))];  lgnd{j} = sprintf('c_%i ',(j-1));
        end
    end
    legend(lgnd,'Location','northeast');  ylabel(sprintf('Coef. of x%i',k));  hold off; 
    if k==K;  xlabel('\delta'); end
    
    
    subplot(K,2,2+(k-1)*2);  % plot residual 
    loglog(delta_array, res_array,'-v','Linewidth',1); hold on;    
    ind = 2:L_gapseq-2;
    x     = log(delta_array); y = log(res_array);
    [a,~] = polyfit(x(ind),y(ind),1);
     if k==K;  xlabel('\delta'); end
    if normalizeBYdeltat == 1    % remove deletat normalizing, to match the new draft
        ylabel(['\delta\sigma_\eta ',sprintf('x%i',k)]);
    lgnd  = ['log (\delta\sigma_\eta) ', sprintf('= %2.2f', a(1)),'log(\delta)',sprintf('+ %2.2f', a(2))];
    else
        ylabel(['\sigma_\eta ',sprintf('x%i',k)]);
        lgnd  = ['log (\sigma_\eta) ', sprintf('= %2.2f', a(1)),'log(\delta)',sprintf('+ %2.2f', a(2))];
    end
    y_ref  =  0.5*(x(ind)-x(ind(end))) + y(ind(end));   loglog(delta_array(ind), exp(y_ref),'k--','linewidth',1);   % the plot is log-log, 
    lgnd  = {lgnd,'Reference slope = 0.5'};
    legend(lgnd,'Location','best');
    hold on; 
end
hold off
set_positionFonts;
figname = [rmInfo.fig_path, 'conv_',rmInfo.gapStr,'_lseEntrywise'];
savefig(gcf,[figname,'.fig']); 
set(gcf,'Position',  [100, 1000, 800, 300+100*K]);tightfig;
print([figname,'.pdf'],'-dpdf');
print([figname,'.eps'],'-depsc');
end
