function [TV_RMvsTrue,TV_RM0vsTrue] = compareStats3(xrm,Xtrue,xrm0,timeLag,deltat,rmInfo,scheme) 
%  compare the PDF and ACF 

IS_scheme = sprintf('IS-%s',rmInfo.rm_type); 
figpath     = rmInfo.fig_path; 
gapStr      = rmInfo.nameStrFn(rmInfo.obsgap); 
figname0    = ['fig',gapStr]; 
figDataFile = [rmInfo.rm_path,'figData_',gapStr,'.mat'];
figname     = [figpath,figname0,'.pdf']; 
if rmInfo.lse_entrywise ==1
    figDataFile = [rmInfo.rm_path,'figData_',gapStr,'_lseEntrywise.mat'];
    figname     = [figpath,figname0,'_lseEntrywise.pdf']; 
end
    
%% compute the PDF and ACF 
if exist(figDataFile,'file')
    load(figDataFile,'ACF_recomputed');
    if ~exist('ACF_recomputed','var');  ACF_recomputed = 0;    end
end
if ~exist(figDataFile,'file') || ACF_recomputed == 0 
    corr_true   = temporal_corr(Xtrue,timeLag);
    xrm_corr    = temporal_corr(xrm,timeLag);
    xrm0_corr   = temporal_corr(xrm0,timeLag);
   [fiTrue,xiTrue] = ksdensity(real(Xtrue(1,:)));  
   [fiRM,~]     = ksdensity(real(xrm(1,:)),xiTrue);
   [fiRM0,~]    = ksdensity(real(xrm0(1,:)),xiTrue);
   ACF_recomputed = 1; 
   save(figDataFile,'corr_true','xrm0_corr','xrm_corr','timeLag','deltat','xiTrue','fiTrue','fiRM','fiRM0','ACF_recomputed');
else 
    load(figDataFile);
end

%% compute the total variation
%{
compute the total variation distance between two probability
distributions, using proposition 1.2 from reference
https://ece.iisc.ac.in/~parimal/2019/statphy/lecture-14.pdf
%}
% compute the F(x)=\int^x p(s) ds for eacth observatin, where p(s) is the pdf via simple Rieman sum
FiTrue=zeros(size(xiTrue)); FiRM=FiTrue; FiRM0=FiTrue;
L_xiTrue=length(xiTrue);
%% Treate each fiRM*dx, fiTrue*dx, fiRM0*dx as discrete pmf on the configuration space xiTrue
FiTrue(2:L_xiTrue)=(fiTrue(1:L_xiTrue-1).*(xiTrue(2:L_xiTrue)-xiTrue(1:L_xiTrue-1)) );
FiRM(2:L_xiTrue)=(fiRM(1:L_xiTrue-1).*(xiTrue(2:L_xiTrue)-xiTrue(1:L_xiTrue-1)) );
FiRM0(2:L_xiTrue)=(fiRM0(1:L_xiTrue-1).*(xiTrue(2:L_xiTrue)-xiTrue(1:L_xiTrue-1)));

% compute the total variation distance
TV_RMvsTrue=1/2*sum(abs(FiRM-FiTrue));
TV_RM0vsTrue=1/2*sum(abs(FiRM0-FiTrue));
   
% plot the results
lgnd = {IS_scheme,'True',scheme}; 
%% compare the stats of the reduced model with the data
tN = length(xrm); tInd =[1:200,101:ceil(tN/200):tN]; tt = tInd*deltat; 
% compare trajectory
figure;
subplot(131);  plot(tt, xrm(tInd),'b');  hold on; 
      plot(tt, Xtrue(tInd),'k-.', tt, xrm0(tInd),'r:'); 
      axis([tt(1)-0.5,tt(end)+0.5, min(Xtrue(tInd))*1.2,max(Xtrue(tInd))*1.5]);
   legend(lgnd); title('Trajectory of x1'); xlabel('Time t'); ylabel('x1'); 
%   figname = [figpath,figname0,'_traj']; print([figname,'.eps'],'-depsc');
   
% compare pdf
subplot(132); 
   h = plot(xiTrue,fiRM,'b',xiTrue,fiTrue,'k-.',xiTrue,fiRM0,'r:'); set(h,{'LineWidth'},{1})
   axis([min(xiTrue),max(xiTrue), min(fiTrue),max(fiTrue)*1.5]); 
   legend(lgnd);
   xlabel('x1'); ylabel('PDF'); title('PDF of x1'); 
  % legend('Reproduced','True'); 
   
% compare correlation
tt = deltat*(0:timeLag);
subplot(133); 
   plot(tt,xrm_corr/xrm_corr(1,1),'b','linewidth',1); hold on; 
   plot(tt,corr_true/corr_true(1,1),'k-.','linewidth',1); 
   plot(tt,xrm0_corr/xrm0_corr(1,1),'r:','linewidth',1);
   title('Correlation of x1'); xlabel('Time Lag t'); ylabel('ACF'); legend(lgnd);
%   figname = [figpath,figname0,'_acf']; print(figname,'-depsc');

set_positionFonts;
 set(gcf,'Position',  [100, 1000, 1200, 400]);tightfig;
print(figname,'-dpdf');
end

