function [data_orig,X,blowupFlag] = forwardEuler(sysInfo,fmDataInfo,X_init,dWdt)
% solve the system by Euler-Maruyama integrator: 
%{ 
        dX = f(X)dt + b(X)dW 
 With  
     X(:,n+1) = X(:,n)+ f(Xn)*dt + b*sqrt(dt) * dW
 Output
    data_orig  
%}
dimX        = sysInfo.dimX; 
drift_neg   = sysInfo.gradV;   
diffMat     = sysInfo.diffusion; 

dt    =   fmDataInfo.dt;  
tN    =   fmDataInfo.tN;
if nargin <3
    X_init =   fmDataInfo.getInitial(1);
end
if ~exist('dWdt','var')  
    dWdt   =   sqrt(dt)*randn(dimX,tN);  % i.i.d increments of Bm for the integrator.
end

X      = zeros(dimX,tN+1);
X(:,1) = X_init;
x0     = X_init;
blowupFlag = 0; i = 1; 
while i <tN
    if norm(x0)>1e5; fprintf('Forward Euler blows up at time= %i\n', i); blowupFlag=1; break;  end
    x1       = x0 - drift_neg(x0)*dt + diffMat*dWdt(:,i); 
    X(:,i+1) = x1; x0 = x1; 
    i  = i+1; 
end

data_orig = struct( ...
    'X',        X,...
    'dWdt',     dWdt, ...
    'tN',       tN, ...
    'dt',       dt ...
);
end

