function [data_orig,X] = schemeEuler(sysInfo,fmDataInfo,X_init,dWdt)
% solve the system by Euler-Maruyama integrator: 
%     dx = - dVx(x,y)dt + sigma_x*dW_x
%     dy = - dVy(x,y)dt + sigms_y*dW_y
% With X = (x,y), the discrete 
%     X(:,n+1) = X(:,n) - [dVx,dVy](Xn)*dt + sigma*sqrt(dt) * dW
% Output
%    data_orig  

dVx     =  fmDataInfo.dVx; 
dVy     =  fmDataInfo.dVy; 
                    
sigma_x  =  sysInfo.sigma_1; 
sigma_y  =  sysInfo.sigma_2; 


dt    =   fmDataInfo.dt;  
tN    =   fmDataInfo.tN;
if nargin <3
    X_init =   fmDataInfo.getInitial(1);
end
if ~exist('dWdt','var')  
    dWdt   =   sqrt(dt)*[sigma_x; sigma_y].*randn(2,tN);  % i.i.d increments of Bm for the integrator.
end

X      =   zeros(2,tN+1);
X(:,1) =   X_init;

for i     =   1:tN
    x        =   X(1,i); 
    y        =   X(2,i);
    X(:,i+1) =   X(:,i) - [dVx(x,y);dVy(x,y)]*dt+dWdt(:,i);
end

% dWdt = dWdt / dt;  % This stores the white noise. 

data_orig = struct( ...
    'X',        X,...
    'dWdt',     dWdt, ...
    'tN',       tN, ...
    'dt',       dt, ...
    'x_range',  [min(X(1,:)), max(X(1,:))], ...
    'y_range',  [min(X(2,:)), max(X(2,:))] ...
);
end

